import rclpy
from behavior_interface.msg import BehaviorStatus
from rclpy.node import Node
from std_msgs.msg import Empty
from .auto_off import AutoOffBehavior
from .float_setter import FloatSetter

class BaseBehavior(Node):
    def __init__(self, name):
        super().__init__('base_behavior_' + name)
        self.name = name
        self.status = False

        self.subscription = self.create_subscription(
            BehaviorStatus,
            'behavior',
            self.listener_callback,
            10
        )

        self.status_publisher = self.create_publisher(
            BehaviorStatus,
            'behaviors_status',
            10
        )
        self.get_logger().info(f'{self.name} behavior node initialized')

    def listener_callback(self, msg):
        if msg.name == 'ping':
            self.publish_status()
        elif msg.name == self.name:
            self.status = msg.status
            self.on_status()

    def publish_status(self):
        status_msg = BehaviorStatus()
        status_msg.name = self.name
        status_msg.status = self.status
        self.status_publisher.publish(status_msg)

    def send_status(self):
        status_msg = BehaviorStatus()
        status_msg.name = self.name
        status_msg.status = self.active
        self.status_publisher.publish(status_msg)

    def on_status(self):
        pass

