import rclpy
from rclpy.node import Node
from .TakeOff import Takeoff
from .MoveLeft import MoveLeft
from .Hover import Hover
from .MoveForward import MoveForward
from .MoveRight import MoveRight
from .StopAndLand import StopAndLand

def main(args=None):
    rclpy.init(args=args)

    takeoff_behavior = Takeoff()
    move_left_behavior = MoveLeft()
    hover_behavior = Hover()
    move_up_behavior = MoveForward()
    move_right_behavior = MoveRight()
    stop_and_land_behavior = StopAndLand()

    executor = rclpy.executors.MultiThreadedExecutor()
    executor.add_node(takeoff_behavior)
    executor.add_node(move_left_behavior)
    executor.add_node(hover_behavior)
    executor.add_node(move_up_behavior)
    executor.add_node(move_right_behavior)
    executor.add_node(stop_and_land_behavior)

    try:
        executor.spin()
    finally:
        takeoff_behavior.destroy_node()
        move_left_behavior.destroy_node()
        hover_behavior.destroy_node()
        move_up_behavior.destroy_node()
        move_right_behavior.destroy_node()
        stop_and_land_behavior.destroy_node()
        rclpy.shutdown()
