import rclpy
from rclpy.node import Node
from behavior_interface.msg import Command
from std_msgs.msg import String
import time

class CommandNode(Node):
    def __init__(self):
        super().__init__('command_node')


        self.behaviors = ['Hover', 'MoveLeft', 'MoveUp', 'MoveRight', 'Takeoff', 'StopAndLand']

        self.commands = {
            'Stop': [(0, 'Hover')],
            'Dance': [(0, 'MoveLeft'), (0, 'MoveUp'), (2.5, 'MoveRight'), (3.2, 'Hover')],
            'TakeOff': [(0, 'Takeoff')],  
            'StopAndLand': [(0, 'StopAndLand')]  
        }


        self.command_subscription = self.create_subscription(
            Command,
            '/joy_teleop',
            self.command_callback,
            10
        )

        self.behavior_publisher = self.create_publisher(String, '/behavior', 10)

        self.event_queue = []
        self.timer = self.create_timer(0.05, self._on_time)

    def command_callback(self, msg):
        command_name = msg.command
        if command_name not in self.commands:
            self.get_logger().error(f"Unknown command: {command_name}")
            return

        self.deactivate_all_behaviors()

        current_time = time.time()
        for delay, behavior_name in self.commands[command_name]:
            event_time = current_time + delay
            self.event_queue.append((event_time, behavior_name))

        self.event_queue.sort()

    def deactivate_all_behaviors(self):
        for behavior in self.behaviors:
            self.publish_behavior(behavior, False)
        self.get_logger().info("All behaviors deactivated")

    def _on_time(self):
        current_time = time.time()
        while self.event_queue and self.event_queue[0][0] <= current_time:
            _, behavior_name = self.event_queue.pop(0)
            self.publish_behavior(behavior_name, True)

    def publish_behavior(self, behavior_name, activate):
        status_msg = String()
        status_msg.data = behavior_name if activate else f"Deactivate {behavior_name}"
        self.behavior_publisher.publish(status_msg)
        self.get_logger().info(f"Behavior {behavior_name} {'activated' if activate else 'deactivated'}")

def main(args=None):
    rclpy.init(args=args)
    command_node = CommandNode()
    rclpy.spin(command_node)
    command_node.destroy_node()
    rclpy.shutdown()
