# External imports
import sqlite3
import cv2
import numpy as np
from vanishing_point import my_line_library as mll  # Your custom library

def load_blob_data(db_path):
    """
    Load and process blob data from a .db3 file.
    
    db_path: str, path to the .db3 file.
    
    Returns:
    List of images decoded from the BLOB data.
    """
    conn = sqlite3.connect(db_path)
    cursor = conn.cursor()
    cursor.execute("SELECT blob_column FROM your_table_name;")  # Adjust the query to match your table/column name
    blobs = cursor.fetchall()

    images = []
    for blob in blobs:
        nparr = np.frombuffer(blob[0], np.uint8)
        img = cv2.imdecode(nparr, cv2.IMREAD_COLOR)
        if img is not None:
            images.append(img)
        else:
            print("Failed to decode image from blob.")
    
    conn.close()
    return images

def process_image(img):
    """
    Process the image using the custom library functions.
    
    img: np.array, input image.
    """
    gray = cv2.cvtColor(img, cv2.COLOR_BGR2GRAY)
    lsd = cv2.createLineSegmentDetector(0)
    lines = lsd.detect(gray)[0]
    lines = mll.from_lsd(lines)

    line_lengths = np.linalg.norm(lines[:, 2:4] - lines[:, 0:2], axis=1)
    lines = lines[line_lengths > 70]  # Filter lines by length

    if lines.size > 0:
        mll.draw_function(img, line_lengths, hmin=50, hmax=img.shape[0] - 50, ymin=np.min(line_lengths), ymax=np.max(line_lengths), color=(0, 255, 255), thickness=2)
        cv2.imshow("Processed Image", img)
        cv2.waitKey(0)
    else:
        print("No lines detected.")

if __name__ == "__main__":
    # Path to your .db3 file
    db_path = '/home/jianjie-chen/uavs/ros_ws/vanishing_point/vanishing_point/door.db3'
    
    # Load images from the .db3 file
    images = load_blob_data(db_path)

    # Process and display each image
    for img in images:
        process_image(img)

    cv2.destroyAllWindows()
